# coding: utf-8

import requests

from .utils import Utils
from .dialog_conversation import DialogConversation
from .dialog_response import DialogResponse

from ..errors import RecastError


class Build:
  def __init__(self, token=None, language=None, proxy=None):
    self.token = token
    self.proxy = proxy
    self.language = language

  def headers(self):
    return {'Authorization': 'Token {}'.format(self.token)}

  def token_required(func):
    def wrapper(self, *args, **kwargs):
      if self.token is None:
        raise ValueError('Token must be set')
      return func(self, *args, **kwargs)
    return wrapper

  @token_required
  def dialog(self, msg, conversation_id, language=None):
    if language is None:
      language = self.language

    params = {'message': msg, 'conversation_id': conversation_id}
    if language is not None:
      params['language'] = language

    response = requests.post('{}/dialog'.format(Utils.BUILD_ENDPOINT), json=params, headers=self.headers())
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message'))
    json = response.json()['results']
    return DialogResponse(json['messages'], json['conversation'], json['nlp'])

  @token_required
  def update_conversation(self, user, bot, conversation_id, opts):
    if 'memory' in opts and type(opts['memory']) is not dict:
      raise ValueError('Memory parameter must be a dict')
    url = '{}/users/{}/bots/{}/builders/v1/conversation_states/{}'.format(Utils.BUILD_ENDPOINT, user, bot, conversation_id)
    response = requests.put(url, json=opts, headers=self.headers())
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message'))
    return DialogConversation(response.json()['results'])

  @token_required
  def delete_conversation(self, user, bot, conversation_id):
    url = '{}/users/{}/bots/{}/builders/v1/conversation_states/{}'.format(Utils.BUILD_ENDPOINT, user, bot, conversation_id)
    response = requests.delete(url, headers=self.headers())
    if response.status_code != requests.codes.no_content:
      raise RecastError(response.json().get('message'))
    return True

  @token_required
  def get_conversation(self, user, bot, conversation_id):
    url = '{}/users/{}/bots/{}/builders/v1/conversation_states/{}'.format(Utils.BUILD_ENDPOINT, user, bot, conversation_id)
    response = requests.get(url, headers=self.headers())
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message'))
    return DialogConversation(response.json()['results'])



# coding: utf-8


class DialogConversation(object):
  def __init__(self, conv):
    self.id = conv.get('conversation_id', conv.get('id', None))
    self.language = conv.get('language', None)
    self.memory = conv['memory']
    self.skill = conv.get('skill', conv.get('last_skill', None))
    self.skill_occurences = conv.get('skill_occurences', conv.get('last_skill_occurences', None))



# coding: utf-8


class Utils():
  """Endpoints"""
  REQUEST_ENDPOINT = 'https://api.recast.ai/v2/request'
  CONVERSE_ENDPOINT = 'https://api.recast.ai/v2/converse'

  """Act constants"""
  ACT_ASSERT = 'assert'
  ACT_COMMAND = 'command'
  ACT_WH_QUERY = 'wh-query'
  ACT_YN_QUERY = 'yn-query'

  """Type constants"""
  TYPE_ABBREVIATION = 'abbr:'
  TYPE_ENTITY = 'enty:'
  TYPE_DESCRIPTION = 'desc:'
  TYPE_HUMAN = 'hum:'
  TYPE_LOCATION = 'loc:'
  TYPE_NUMBER = 'num:'

  """Sentiment constants"""
  SENTIMENT_VPOSITIVE = 'vpositive'
  SENTIMENT_POSITIVE = 'positive'
  SENTIMENT_NEUTRAL = 'neutral'
  SENTIMENT_NEGATIVE = 'negative'
  SENTIMENT_VNEGATIVE = 'vnegative'



# coding: utf-8

from .dialog_message import DialogMessage
from .dialog_conversation import DialogConversation

from ..request.models import Response as NLPResponse


class DialogResponse(object):
  def __init__(self, messages, conversation, nlp):
    if type(messages) is not list:
      raise ValueError('Invalid messages format: {}'.format(messages))
    self.messages = [DialogMessage(msg) for msg in messages]
    self.conversation = DialogConversation(conversation)
    self.nlp = NLPResponse(nlp)



# coding: utf-8

from .intent import Intent
from .entity import Entity

from ..utils import Utils

import json


class Response():
  def __init__(self, response):
    self.raw = json.dumps({'results': response})
    self.uuid = response['uuid']
    self.source = response['source']
    self.intents = [Intent(i) for i in response['intents']]
    self.act = response['act']
    self.type = response['type']
    self.sentiment = response['sentiment']
    self.entities = [Entity(n, ee) for n, e in response['entities'].items() for ee in e]
    self.language = response['language']
    self.processing_language = response['processing_language']
    self.version = response['version']
    self.timestamp = response['timestamp']
    self.status = response['status']

  @property
  def intent(self):
    try:
      return self.intents[0]
    except IndexError:
      return None

  def get(self, name):
    for entity in self.entities:
      if (entity.name.lower() == name.lower()):
        return entity

  def all(self, name):
    entities = []

    for entity in self.entities:
      if (entity.name.lower() == name.lower()):
        entities.append(entity)

    return entities

  @property
  def is_assert(self):
    return self.act == Utils.ACT_ASSERT

  @property
  def is_command(self):
    return self.act == Utils.ACT_COMMAND

  @property
  def is_wh_query(self):
    return self.act == Utils.ACT_WH_QUERY

  @property
  def is_yn_query(self):
    return self.act == Utils.ACT_YN_QUERY

  @property
  def is_abbreviation(self):
    return Utils.TYPE_ABBREVIATION in self.type

  @property
  def is_entity(self):
    return Utils.TYPE_ENTITY in self.type

  @property
  def is_description(self):
    return Utils.TYPE_DESCRIPTION in self.type

  @property
  def is_human(self):
    return Utils.TYPE_HUMAN in self.type

  @property
  def is_location(self):
    return Utils.TYPE_LOCATION in self.type

  @property
  def is_number(self):
    return Utils.TYPE_NUMBER in self.type

  @property
  def is_vpositive(self):
    return self.sentiment == Utils.SENTIMENT_VPOSITIVE

  @property
  def is_positive(self):
    return self.sentiment == Utils.SENTIMENT_POSITIVE

  @property
  def is_neutral(self):
    return self.sentiment == Utils.SENTIMENT_NEUTRAL

  @property
  def is_negative(self):
    return self.sentiment == Utils.SENTIMENT_NEGATIVE

  @property
  def is_vnegative(self):
    return self.sentiment == Utils.SENTIMENT_VNEGATIVE



# coding: utf-8

import io
import requests

from .models import Response
from .utils import Utils

from ..errors import RecastError


class Analyse():
  def analyse_text(self, text, token=None, language=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    language = language or self.language

    body = {'text': text}
    if language is not None:
      body['language'] = language

    response = requests.post(
      Utils.REQUEST_ENDPOINT,
      json=body,
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    return Response(response.json()['results'])

  def analyse_file(self, filename, token=None, language=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    language = language or self.language

    filename = open(filename, 'rb') if not isinstance(filename, io.IOBase) else filename

    body = {'voice': filename}
    if language is not None:
      body['language'] = language

    response = requests.post(
      Utils.REQUEST_ENDPOINT,
      files=body,
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    return Response(response.json()['results'])



# coding: utf-8

import re
import json


class Message():
  def __init__(self, request):
    request = json.loads(request)
    for k, v in request.items():
      setattr(self, re.sub('(?!^)([A-Z])', r'_\1', k).lower(), v)

    self.conversation_id = request['message']['conversation']
    self.content = request['message']['attachment']['content']
    self.type = request['message']['attachment']['type']



# coding: utf-8

import io
import requests

from .models import Conversation
from .utils import Utils

from ..errors import RecastError


class Converse():
  def converse_text(self, text, token=None, language=None, conversation_token=None, memory=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    language = language or self.language

    body = {'text': text}
    if language is not None:
      body['language'] = language
    if conversation_token is not None:
      body['conversation_token'] = conversation_token
    if memory is not None:
      body['memory'] = memory

    response = requests.post(
      Utils.CONVERSE_ENDPOINT,
      json=body,
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    return Conversation(response, token)

  def converse_file(self, filename, token=None, language=None, conversation_token=None, memory=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    language = language or self.language

    filename = open(filename, 'rb') if not isinstance(filename, io.IOBase) else filename

    body = {'voice': filename}
    if language is not None:
      body['language'] = language
    if conversation_token is not None:
      body['conversation_token'] = conversation_token
    if memory is not None:
      body['memory'] = memory

    response = requests.post(
      Utils.CONVERSE_ENDPOINT,
      files=body,
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    return Conversation(response)



# coding: utf-8

import requests

from .action import Action
from .intent import Intent
from .entity import Entity

from ..utils import Utils

from ...errors import RecastError


class Conversation():
  def __init__(self, response, token):
    self.token = token

    self.raw = response.text

    response = response.json()
    response = response['results']

    self.uuid = response['uuid']
    self.source = response['source']
    self.replies = response['replies']
    self.action = Action(response['action']) if response['action'] else None
    self.next_actions = [Action(a) for a in response['next_actions']]
    self.memory = [Entity(n, e) for n, e in response['memory'].items() if e]
    self.sentiment = response['sentiment']
    self.entities = [Entity(n, ee) for n, e in response['entities'].items() for ee in e]
    self.intents = [Intent(i) for i in response['intents']]
    self.conversation_token = response['conversation_token']
    self.language = response['language']
    self.processing_language = response['processing_language']
    self.version = response['version']
    self.timestamp = response['timestamp']
    self.status = response['status']

  @property
  def reply(self):
    try:
      return self.replies[0]
    except IndexError:
      return None

  @property
  def next_action(self):
    try:
      return self.next_actions[0]
    except IndexError:
      return None

  def joined_replies(self, sep=' '):
    return sep.join(self.replies)

  def get_memory(self, key=None):
    if key is None:
      return self.memory

    for entity in self.memory:
      if (entity.name.lower() == key.lower()):
        return entity

    return None

  @property
  def intent(self):
    try:
      return self.intents[0]
    except IndexError:
      return None

  @property
  def is_vpositive(self):
    return self.sentiment == Utils.SENTIMENT_VPOSITIVE

  @property
  def is_positive(self):
    return self.sentiment == Utils.SENTIMENT_POSITIVE

  @property
  def is_neutral(self):
    return self.sentiment == Utils.SENTIMENT_NEUTRAL

  @property
  def is_negative(self):
    return self.sentiment == Utils.SENTIMENT_NEGATIVE

  @property
  def is_vnegative(self):
    return self.sentiment == Utils.SENTIMENT_VNEGATIVE

  def set_memory(self, memory):
    body = {'conversation_token': self.conversation_token, 'memory': memory}
    response = requests.put(
      Utils.CONVERSE_ENDPOINT,
      json=body,
      headers={'Authorization': "Token {}".format(self.token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    response = response.json()
    response = response['results']
    return [Entity(n, e) for n, e in response['memory'].items() if e]

  def reset_memory(self, key=None):
    body = {'conversation_token': self.conversation_token}
    if key:
      body['memory'] = {key: None}
    response = requests.put(
      Utils.CONVERSE_ENDPOINT,
      json=body,
      headers={'Authorization': "Token {}".format(self.token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    response = response.json()
    response = response['results']
    return [Entity(n, e) for n, e in response['memory'].items() if e]

  def reset_conversation(self):
    body = {'conversation_token': self.conversation_token}
    response = requests.delete(
      Utils.CONVERSE_ENDPOINT,
      json=body,
      headers={'Authorization': "Token {}".format(self.token)}
    )
    if response.status_code != requests.codes.ok:
      raise RecastError(response.json().get('message', ''))

    response = response.json()
    response = response['results']
    return [Entity(n, e) for n, e in response['memory'].items() if e]



# coding: utf-8


class DialogMessage(object):
  def __init__(self, msg):
    if type(msg) is not dict or 'type' not in msg or 'content' not in msg:
      raise ValueError('Invalid message format: {}'.format(msg))
    self.type = msg['type']
    self.content = msg['content']



# coding: utf-8

from .response import Response
from .conversation import Conversation
from .action import Action
from .intent import Intent
from .entity import Entity

__all__ = ['Response', 'Conversation', 'Action', 'Intent', 'Entity']



# coding: utf-8

import requests

from .models import Message as Model
from .utils import Utils

from ..errors import RecastError


class Message():
  def parse_message(self, request):
    return Model(request.get_data())

  def send_message(self, messages, conversation_id, token=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    url = Utils.CONVERSATION_ENDPOINT % (conversation_id) + '/messages'
    response = requests.post(
      url,
      json={'messages': messages},
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.created:
      raise RecastError(response.json().get('message'))

    return response

  def broadcast_message(self, messages, token=None):
    token = token or self.token
    if token is None:
      raise RecastError("Token is missing")

    url = Utils.MESSAGE_ENDPOINT % ('')
    response = requests.post(
      url,
      json={'messages': messages},
      headers={'Authorization': "Token {}".format(token)}
    )
    if response.status_code != requests.codes.created:
      raise RecastError(response.json().get('message'))

    return response



# coding: utf-8


class Utils():
  """Versioning"""
  MAJOR = '3'
  MINOR = '0'
  MICRO = '0'
  VERSION = "%s.%s.%s" % (MAJOR, MINOR, MICRO)

  """Endpoints"""
  TRAIN_ENDPOINT = 'https://api.recast.ai/v2/'
  CONNECT_ENDPOINT = 'https://api.recast.ai/connect/v1/'
  HOST_ENDPOINT = 'https://api.recast.ai/host/v1/'
  REQUEST_ENDPOINT = 'https://api.recast.ai/v2/'
  MONITOR_ENDPOINT = 'https://api.recast.ai/monitor/v1/'



# coding: utf-8


class Entity():
  def __init__(self, name, entity):
    self.name = name

    for k, v in entity.items():
      setattr(self, k, v)

  def __repr__(self):
    attributes = []
    for method in dir(self):
      if not method.startswith('__') and method != 'name':
        value = getattr(self, method)
        attributes.append("{}={}".format(method, value))

    return "{} ({})".format(self.name, ', '.join(attributes))



# coding: utf-8

from .apis import Connect, Request, Build


class Client():
  def __init__(self, token=None, language=None, proxy=None):
    for api in [Connect, Request, Build]:
      setattr(self, api.__name__.lower(), api(token=token, language=language, proxy=proxy))



# coding: utf-8

from .message import Message


class Connect(Message):
  def __init__(self, token=None, language=None, proxy=None):
    self.token = token
    self.language = language
    self.proxy = proxy



# coding: utf-8


class Action():
  def __init__(self, action):
    self.slug = action['slug']
    self.done = action['done']
    self.reply = action['reply']

  def __repr__(self):
    attributes = []
    for method in dir(self):
      if not method.startswith('__') and method != 'slug':
        value = getattr(self, method)
        attributes.append("{}={}".format(method, value))

    return "{} ({})".format(self.name, ', '.join(attributes))
